Bases = {}

local timer = 1
local grid_size = 128
local grids_X = window_width / grid_size
local grids_Y = window_height / grid_size
local pulse = -.05
local bases = {}
local lines = {}
local attacks = {}

function Bases:load()
  for y=2, grids_Y do
    for x=1, grids_X do
      Bases:createBase(x, y) -- create all of the bases for the game grids
    end 
  end  
  Bases:createNeighbors()
end

function Bases:update(dt)
  for _,base in ipairs(bases) do    
    Bases:getHovering(base.id, tx, ty) -- mouse over a base?
  end
end

function Bases:draw()
  love.graphics.setLineWidth(1)
  for _,base in ipairs(bases) do      
    if base.owner == 1 then -- draw player's base(s) and neighbors
      love.graphics.setColor(128,128,128,64)
      for k,line in ipairs(base.lines) do
        love.graphics.line(lines[line].shape:getPoints()) -- draw lines to neighbors
      end
      Bases:drawBase(base.id)
      for i,j in ipairs(base.neighbors) do -- find and draw the neighboring bases
        love.graphics.setLineWidth(bases[j].lineWidth)
        Bases:drawBase(bases[j].id)
      end
    end
    if base.selectedUnits > 0 then love.graphics.print(base.selectedUnits, love.mouse.getX() + 7 - tx, love.mouse.getY() - 7 -ty) end -- draw the attacking units on top
  end   
end

function Bases:createBase(x, y)
  randomX = love.math.random(24, grid_size - 24)
  randomY = love.math.random(24, grid_size - 24)
  n = #bases + 1
  bases[n] = {}
  bases[n].id = n
  bases[n].hovering = false
  bases[n].selected = false
  bases[n].specialty = {}
  bases[n].units = 100
  bases[n].selectedUnits = 0
  bases[n].attacking = false
  bases[n].lineWidth =1
  bases[n].owner = 0
  
  bases[n].defenseMod = 1
  bases[n].offenseMod = 1
  bases[n].timer = 1
  
  bases[n].neighbors = {}
  bases[n].lines = {}
  bases[n].color = {128,128,128,255}
  bases[n].secondaryColor = {64,64,64,255}
  bases[n].body =  love.physics.newBody(world, (x * grid_size) - grid_size + randomX, (y * grid_size) - grid_size + randomY, "dynamic")
  bases[n].shape = love.physics.newCircleShape(10)
  bases[n].fixture = love.physics.newFixture(bases[n].body, bases[n].shape)
end

function Bases:createNeighbors()
  n = #bases
  for n = n, 1, -1 do
    for i = 1, #bases, 1 do
      local distance = Utils:distance(bases[n].body:getX(), bases[n].body:getY(), bases[i].body:getX(), bases[i].body:getY())
      if distance < 180 and i ~= n then -- how close do they need to be to be considered a "neighbor" and attackable?
        x = #lines + 1
        lines[x] = {}
        lines[x].body = love.physics.newBody(world, bases[n].body:getX(), bases[n].body:getY(), "dynamic")
        lines[x].shape = love.physics.newChainShape(false, bases[n].body:getX(), bases[n].body:getY(), bases[i].body:getX(), bases[i].body:getY())
        lines[x].fixture = love.physics.newFixture(lines[x].body, lines[x].shape, 1)
        table.insert(bases[i].lines,1,x)
        table.insert(bases[n].neighbors,1,i)
      end
    end
  end
end

function Bases:getHovering(id, tx, ty)
  local mouseOver = bases[id].shape:testPoint( bases[id].body:getX(), bases[id].body:getY(), 0, love.mouse.getX() - tx, love.mouse.getY() - ty )
  if mouseOver then
    bases[id].hovering = true
    if bases[id].lineWidth < 1 or bases[id].lineWidth > 2 then pulse = pulse * -1 end -- pulsating visual effect
    bases[id].lineWidth = bases[id].lineWidth + pulse
  else
    bases[id].hovering = false
  end
end

function Bases:setSelectedBase(id)
  bases[selectedBase].selected = false -- set the old one to not selected first
  selectedBase = id
  bases[id].selected = true
end

function Bases:getSelectedBase()
  return bases[selectedBase]
end

function Bases:drawBase(id)  
  if bases[id].selected == true and bases[id].owner ~= 0 then
    love.graphics.setColor(bases[id].secondaryColor)
  else
    love.graphics.setColor(bases[id].color)
  end
  love.graphics.circle("fill", bases[id].body:getX(), bases[id].body:getY(), 10 + bases[id].units * .03)
  love.graphics.setColor(255,255,255,255)
  if bases[id].units > 10 then 
    love.graphics.circle("fill", bases[id].body:getX() + 12 + bases[id].units * .03, bases[id].body:getY(), 2)
  end
  if DEBUG or bases[id].owner == 1 then
    love.graphics.printf(bases[id].units, bases[id].body:getX() - 16, bases[id].body:getY()- 6, 34, "center")
  end
  
  local buttons = UI:getButtons()
  for i=1, #bases[id].specialty do 
    local specialty = bases[id].specialty[i]
    love.graphics.draw(buttons[specialty].image,bases[id].body:getX() + i * 20, bases[id].body:getY() - 25, 0, .33)
  end  
  love.graphics.setColor(255,255,255,255)
  love.graphics.circle("line", bases[id].body:getX(), bases[id].body:getY(), 10 + bases[id].units * .03 , 20 )
end

function Bases:getBases()
  return bases
end

return Bases